<?php
// Remplir ces champs avec les valeurs obtenues sur AriseID Connect
$clientId = getenv('AIDC_CLIENT_ID') ?: 'be28aa62-643f-42b4-80ae-902d45f93331';
$clientSecret = getenv('AIDC_CLIENT_SECRET') ?: 'secret-gimydg6n7ddn6qwen6ypvccmrc4kpccul6l3cmdh4zbym4hwghrq';

$oauthURLBase = getenv('OAUTH_SERVER') ?: 'https://oidc.iiens.net';
// L'URL à laquelle on enverra d'abord l'utilisateur pour obtenir son autorisation
$authorizeURL = "{$oauthURLBase}/oauth2/auth";
// Le point d'accès à partir duquel notre serveur demandera un jeton d'accès
$tokenURL = "{$oauthURLBase}/oauth2/token";

// L'URL racine à utiliser pour effectuer des demandes d'API authentifiées
$apiURLBase = getenv('API_SERVER') ?: 'https://api.iiens.net';

// L'URL de ce script, utilisé comme URL de redirection
$baseURL = (getenv('DOKKU_PROXY_SSL_PORT') ? 'https://' : 'http://') . $_SERVER['SERVER_NAME'] . ($_SERVER['SERVER_PORT'] != '80' ? ':' . $_SERVER['SERVER_PORT'] : '') . $_SERVER['PHP_SELF'];

// On lance une session afin d'avoir un endroit où stocker les données entre les redirections
session_start();

// Commence le processus de connexion en envoyant l'utilisateur
// à la page d'autorisation d'AriseID Connect
if (isset($_GET['action']) && $_GET['action'] == 'login') {
  unset($_SESSION['access_token']);

  // Génère un hash aléatoire et le stocke dans la session
  $_SESSION['state'] = bin2hex(random_bytes(16));

  $params = [
    'response_type' => 'code',
    'client_id' => $clientId,
    'redirect_uri' => $baseURL,
    'scope' => 'profile',
    'state' => $_SESSION['state']
  ];

  // Redirige l'utilisateur vers la page d'autorisation d'AriseID Connect
  header('Location: ' . $authorizeURL . '?' . http_build_query($params));
  die();
}

if (isset($_GET['action']) && $_GET['action'] == 'logout') {
  unset($_SESSION['access_token']);
  header("Location: $baseURL");
  die();
}

// Quand AriseID Connect redirige l'utilisateur ici, il y aura
// un paramètre "code" et "state" dans la chaîne de requête
if (isset($_GET['code'])) {
  // Certifie que l'état correspond à l'état stocké
  if (!isset($_GET['state']) || $_SESSION['state'] != $_GET['state']) {

    header("Location: $baseURL?error=invalid_state");
    die();
  }

  // Échange le code d'authentification contre un jeton d'accès
  $ch = curl_init($tokenURL);
  curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
  curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query([
    'grant_type' => 'authorization_code',
    'client_id' => $clientId,
    'client_secret' => $clientSecret,
    'redirect_uri' => $baseURL,
    'code' => $_GET['code']
  ]));
  $response = curl_exec($ch);
  $tokens = json_decode($response, true);

  $_SESSION['access_token'] = $tokens['access_token'];

  header("Location: $baseURL");
  die();
}

function graphql_request($apiURLBase, $query)
{
  $endpoint = "$apiURLBase/graphql/v0";

  $headers = [];
  $headers[] = 'Content-Type: application/json';
  $headers[] = 'Authorization: Bearer ' . $_SESSION['access_token'];

  $ch = curl_init();

  curl_setopt($ch, CURLOPT_URL, $endpoint);
  curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
  curl_setopt($ch, CURLOPT_POSTFIELDS, $query);
  curl_setopt($ch, CURLOPT_POST, 1);
  curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

  $response = curl_exec($ch);

  if (curl_errno($ch)) {
    echo 'Error:' . curl_error($ch);
  }

  return json_decode($response, true);
}

$profile_query = '{"query":"{ profile { id name civilName givenName middleName forenames familyName nickname preferredUsername preferredNickname gender birthdate profile picture { url } website zoneinfo locale updatedAt promotion year groups { role group { name } } } }"}';

$jsonFlags = JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES;

?>
<html lang="fr">

<head>
  <meta charset="utf-8">
  <meta name="viewport" content="width=device-width, initial-scale=1">
  <meta name="color-scheme" content="light dark">
  <link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/@picocss/pico@2/css/pico.classless.min.css">
  <title>OAuth2 Demo</title>
</head>

<body>
  <main>
    <nav>
      <ul>
        <li>
          <h2>OAuth2 Demo</h2>
        </li>
      </ul>
      <ul>
        <li>
          <a href="/"> Retour</a>
        </li>
        <li>
          <a target="_blank" href="https://git.iiens.net/arise/ariseid-connect/exemples/php">
            Code source
          </a>
        </li>
      </ul>
    </nav>
    <p></p>
    <?php if (empty($_SESSION['access_token'])): ?>
      <h3>Non connecté</h3>
      <a href="?action=login"><button>Se connecter</button></a>
    <?php else: ?>
      <!-- S'il y a un jeton d'accès dans la session l'utilisateur est connecté -->
      <h3>Connecté</h3>
      <a href="?action=logout"><button>Se déconnecter</button></a>

      <h3>Profil (via API)</h3>
      <pre><code>
<?php echo json_encode(graphql_request($apiURLBase, $profile_query), $jsonFlags); ?>
      </code></pre>
    <?php endif; ?>
  </main>
</body>