<?php
// Remplir ces champs avec les valeurs obtenues sur AriseID Connect
$clientId = getenv('AIDC_CLIENT_ID') ?: 'be28aa62-643f-42b4-80ae-902d45f93331';
$clientSecret = getenv('AIDC_CLIENT_SECRET') ?: 'secret-gimydg6n7ddn6qwen6ypvccmrc4kpccul6l3cmdh4zbym4hwghrq';

$oauthURLBase = getenv('OAUTH_SERVER') ?: 'https://oidc.iiens.net';
// L'URL à laquelle on enverra d'abord l'utilisateur pour obtenir son autorisation
$authorizeURL = "{$oauthURLBase}/oauth2/auth";
// Le point d'accès à partir duquel notre serveur demandera un jeton d'accès
$tokenURL = "{$oauthURLBase}/oauth2/token";
// Le point d'accès à partir duquel notre serveur demandera un jeton d'identité
$userinfoURL = "{$oauthURLBase}/userinfo";

// L'URL racine à utiliser pour effectuer des demandes d'API authentifiées
$apiURLBase = getenv('API_SERVER') ?: 'https://api.iiens.net';

// L'URL de ce script, utilisé comme URL de redirection
$baseURL = (getenv('DOKKU_PROXY_SSL_PORT') ? 'https://' : 'http://') . $_SERVER['SERVER_NAME'] . ($_SERVER['SERVER_PORT'] != '80' ? ':' . $_SERVER['SERVER_PORT'] : '') . $_SERVER['PHP_SELF'];

// On lance une session afin d'avoir un endroit où stocker les données entre les redirections
session_start();

// Commence le processus de connexion en envoyant l'utilisateur
// à la page d'autorisation d'AriseID Connect
if (isset($_GET['action']) && $_GET['action'] == 'login') {
  unset($_SESSION['access_token']);

  // Génère un hash aléatoire et le stocke dans la session
  $_SESSION['state'] = bin2hex(random_bytes(16));

  $params = [
    'response_type' => 'code',
    'client_id' => $clientId,
    'redirect_uri' => $baseURL,
    'scope' => 'openid profile email',
    'state' => $_SESSION['state']
  ];

  // Redirige l'utilisateur vers la page d'autorisation d'AriseID Connect
  header('Location: ' . $authorizeURL . '?' . http_build_query($params));
  die();
}

if (isset($_GET['action']) && $_GET['action'] == 'logout') {
  unset($_SESSION['user_id']);
  header("Location: $baseURL");
  die();
}

// Quand AriseID Connect redirige l'utilisateur ici, il y aura
// un paramètre "code" et "state" dans la chaîne de requête
if (isset($_GET['code'])) {
  // Certifie que l'état correspond à l'état stocké
  if (!isset($_GET['state']) || $_SESSION['state'] != $_GET['state']) {

    header("Location: $baseURL?error=invalid_state");
    die();
  }

  // Échange le code d'authentification contre un jeton d'accès
  $ch = curl_init($tokenURL);
  curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
  curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query([
    'grant_type' => 'authorization_code',
    'client_id' => $clientId,
    'client_secret' => $clientSecret,
    'redirect_uri' => $baseURL,
    'code' => $_GET['code']
  ]));
  $response = curl_exec($ch);
  $tokens = json_decode($response, true);

  // Note : Il est préférable d'utiliser une vraie bibliothèque JWT,
  // mais cette solution peut faire l'affaire en cas de besoin.
  // L'opération n'est sûre que parce que le jeton d'identification
  // provient de la connexion https d'AriseID Connect et non
  // d'une redirection de navigateur non fiable.

  // Sépare la chaîne JWT en trois parties
  $jwt = explode('.', $tokens['id_token']);

  // Extrait la partie centrale, la décode en base64, puis la décode en json
  $userinfo = json_decode(base64_decode($jwt[1]), true);

  $_SESSION['user_id'] = $userinfo['sub'];
  $_SESSION['email'] = $userinfo['email'];

  // Stocke le jeton d'accès et le jeton d'identification
  // pour pouvoir les utiliser ultérieurement
  $_SESSION['access_token'] = $tokens['access_token'];
  $_SESSION['id_token'] = $tokens['id_token'];
  $_SESSION['userinfo'] = $userinfo;

  header("Location: $baseURL");
  die();
}

function userinfo_request($userinfoURL)
{
  $ch = curl_init($userinfoURL);
  curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
  curl_setopt($ch, CURLOPT_HTTPHEADER, [
    'Authorization: Bearer ' . $_SESSION['access_token']
  ]);
  $response = curl_exec($ch);

  if (curl_errno($ch)) {
    echo 'Error:' . curl_error($ch);
  }

  return json_decode($response, true);
}

$jsonFlags = JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES;

?>
<html lang="fr">

<head>
  <meta charset="utf-8">
  <meta name="viewport" content="width=device-width, initial-scale=1">
  <meta name="color-scheme" content="light dark">
  <link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/@picocss/pico@2/css/pico.classless.min.css">
  <title>OpenID Connect Demo</title>
</head>

<body>
  <main>
    <nav>
      <ul>
        <li>
          <h2>OpenID Connect Demo</h2>
        </li>
      </ul>
      <ul>
        <li>
          <a href="/"> Retour</a>
        </li>
        <li>
          <a target="_blank" href="https://git.iiens.net/arise/ariseid-connect/exemples/php">
            Code source
          </a>
        </li>
      </ul>
    </nav>
    <p></p>
    <?php if (empty($_SESSION['user_id'])): ?>
      <h3>Non connecté</h3>
      <a href="?action=login"><button>Se connecter</button></a>
    <?php else: ?>
      <!-- S'il y a un ID utilisateur dans la session l'utilisateur est connecté -->
      <h3>Connecté</h3>
      <p>User ID: <code><?php echo $_SESSION['user_id']; ?></code></p>
      <p>Email: <code><?php echo $_SESSION['email']; ?></code></p>
      <a href="?action=logout"><button>Se déconnecter</button></a>

      <h3>ID Token</h3>
      <pre><code>
<?php echo json_encode($_SESSION['userinfo'], $jsonFlags); ?>
      </code></pre>

      <h3>User Info</h3>
      <pre><code>
<?php echo json_encode(userinfo_request($userinfoURL), $jsonFlags); ?>
      </code></pre>
    <?php endif; ?>
  </main>
</body>